/*
 * MiniPC board late initialisation, based on uboot-xburst and xburst-tools.
 *
 * Copyright (C) 2000-2009 Wolfgang Denk, DENX Software Engineering, <wd@denx.de>
 * Copyright (C) 2005-2006 Ingenic Semiconductor, <jlwei@ingenic.cn>
 * Copyright (C) Xiangfu Liu <xiangfu.z@gmail.com>
 * Copyright (C) 2015, 2016 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "board.h"
#include "minipc.h"

/* Later initialisation functions. */

void gpio_init2()
{
	/* LED enable */
        __gpio_as_output(GPIO_LED_EN);
        __gpio_set_pin(GPIO_LED_EN);

	__harb_usb0_uhc();
	__gpio_as_emc();
	__gpio_as_dma();

	/*
	 * Initialize LCD pins
	 */
	__gpio_as_lcd_master();

	/*
	 * Initialize MSC pins
	 */
	__gpio_as_msc();
}

void cpm_init()
{
	__cpm_stop_all();
}

void rtc_init()
{
	/* NOTE: May only be accessible via I2C. */
}

/* Timer routines. */

unsigned long timestamp;
unsigned long lastdec;

/*
 * Timer without interrupts.
 */

int timer_init()
{
        __ost_set_clock(TIMER_CHAN, OST_TCSR_CKS_PCLK_256);
        __ost_set_reload(TIMER_CHAN, TIMER_FDATA);
        __ost_set_count(TIMER_CHAN, TIMER_FDATA);
        __ost_enable_channel(TIMER_CHAN);

	__cpm_start_ost();

        lastdec = TIMER_FDATA;
        timestamp = 0;

        return 0;
}

/* Timer interrupt activation. */

void timer_init_irq()
{
	__ost_enable_interrupt(TIMER_CHAN);
	/* NOTE: Need flag clearing? */
	__intc_unmask_irq(TIMER_CHAN_IRQ);
}

void timer_clear()
{
	__intc_ack_irq(TIMER_CHAN_IRQ);
	__ost_clear_uf(TIMER_CHAN);
}

/* GPIO interrupt activation. */

void gpio_init_irq()
{
	unsigned short i;

	for (i = 0; i < 7; i++) {
		__gpio_as_irq_low_level(GPIO_KEYIN_BASE + i);
	}

	__gpio_as_irq_low_level(GPIO_POWER);
	__intc_unmask_irq(GPIO_IRQ);
}

int gpio_have_irq(u8 gpio)
{
	return (REG_GPIO_GPFR(gpio / 32) & (1 << (gpio % 32)));
}

/* Board startup detection. */

int is_started()
{
	return REG_CPM_MSCR != 0;
}
