/*
 * Interrupt handling.
 *
 * Copyright (C) 2015, 2016 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "board-defs.h"

#include "init.h"
#include "board.h"
#include "cpu.h"
#include "cpu_op.h"
#include "irq.h"
#include "tasks.h"

/* Initialisation and handling. */

void irq_init()
{
	handle_error_level();
	timer_init_irq();
	gpio_init_irq();
	init_interrupts();
}

void irq_handle()
{
	/* Check interrupt identity. */

	if (REG_INTC_IPR & (1 << TIMER_CHAN_IRQ)) {

		/* Clear interrupt status. */

		timer_clear();
	}

	/* GPIO interrupts. */

	if (REG_INTC_IPR & (1 << GPIO_IRQ)) {

		/* Check for the power button. */

		if (gpio_have_irq(GPIO_POWER)) {

			/* Switch task. */

			switch_task();

			/* Clear interrupt status. */

			gpio_clear(GPIO_POWER);
		}
	}

	/* Handle other interrupts, anyway. */

	irq_clear();
}
