/*
 * U-Boot and JzRISC LCD controller definitions
 *
 * Copyright (C) 2001 Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 * Copyright (C) 2005-2007, Ingenic Semiconductor Inc.
 * Copyright (C) Xiangfu Liu <xiangfu@sharism.cc>
 * Copyright (C) 2015, 2016 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#ifndef __JZLCD_H__
#define __JZLCD_H__

#include "xburst_types.h"

void lcd_set_bpp(u8 bpp);
unsigned long lcd_ctrl_init();
void lcd_enable();
void lcd_disable();

/*
 * Framebuffer characteristics
 */
struct jzfb_info {
	unsigned int cfg;	/* panel mode and pin usage etc. */
	unsigned int w;
	unsigned int h;
	unsigned int bpp;	/* bit per pixel */
	unsigned int fclk;	/* frame clk */
	unsigned int hsw;	/* hsync width, in pclk */
	unsigned int vsw;	/* vsync width, in line count */
	unsigned int elw;	/* end of line, in pclk */
	unsigned int blw;	/* begin of line, in pclk */
	unsigned int efw;	/* end of frame, in line count */
	unsigned int bfw;	/* begin of frame, in line count */
};

/*
 * LCD controller stucture for JZSOC: JZ4740
 */
struct jz_fb_dma_descriptor {
	struct jz_fb_dma_descriptor *fdadr;	/* Frame descriptor address register */
	unsigned long fsadr;			/* Frame source address register */
	unsigned long fidr;			/* Frame ID register */
	unsigned long ldcmd;			/* Command register */
};

/*
 * Jz LCD info
 */
struct jz_fb_info {

	struct jz_fb_dma_descriptor *fdadr0;	/* physical address of frame/palette descriptor */
	struct jz_fb_dma_descriptor *fdadr1;	/* physical address of frame descriptor */

	/* DMA descriptors */
	struct jz_fb_dma_descriptor *dmadesc_fblow;
	struct jz_fb_dma_descriptor *dmadesc_fbhigh;
	struct jz_fb_dma_descriptor *dmadesc_palette;

	unsigned long screen;		/* address of frame buffer */
	unsigned long palette;		/* address of palette memory */
};

/*
 * Concise display characteristics with low-level structure reference
 */
typedef struct vidinfo {
	unsigned short vl_col;		/* Number of columns (i.e. 640) */
	unsigned short vl_row;		/* Number of rows (i.e. 480) */
	unsigned char vl_bpix;		/* Bits per pixel, 0 = 1, 1 = 2, 2 = 4, 3 = 8 */

	struct jz_fb_info jz_fb;
} vidinfo_t;

/* Alignment/rounding macros. */

#define ALIGN(x,a)              __ALIGN_MASK((x),(typeof(x))(a)-1)
#define __ALIGN_MASK(x,mask)    (((x)+(mask))&~(mask))

/* General values for colour depths and framebuffer characteristics. */

#define LCD_MONOCHROME	0
#define LCD_COLOR2	1
#define LCD_COLOR4	2
#define LCD_COLOR8	3
#define LCD_COLOR16	4
#define LCD_COLOR32	5

/* Calculate number of bits per pixel and number of colours. */

#define NBITS(bit_code)		(1 << (bit_code))
#define NCOLORS(bit_code)	(1 << NBITS(bit_code))

/* Transfer and display types. */

#define MODE_MASK		0x0f
#define MODE_TFT_GEN		0x00
#define MODE_TFT_SHARP		0x01
#define MODE_TFT_CASIO		0x02
#define MODE_TFT_SAMSUNG	0x03
#define MODE_CCIR656_NONINT	0x04
#define MODE_CCIR656_INT	0x05
#define MODE_STN_COLOR_SINGLE	0x08
#define MODE_STN_MONO_SINGLE	0x09
#define MODE_STN_COLOR_DUAL	0x0a
#define MODE_STN_MONO_DUAL	0x0b
#define MODE_8BIT_SERIAL_TFT    0x0c

#define MODE_TFT_18BIT          (1<<7)

#define STN_DAT_PIN1	(0x00 << 4)
#define STN_DAT_PIN2	(0x01 << 4)
#define STN_DAT_PIN4	(0x02 << 4)
#define STN_DAT_PIN8	(0x03 << 4)
#define STN_DAT_PINMASK	STN_DAT_PIN8

#define STFT_PSHI	(1 << 15)
#define STFT_CLSHI	(1 << 14)
#define STFT_SPLHI	(1 << 13)
#define STFT_REVHI	(1 << 12)

#define SYNC_MASTER	(0 << 16)
#define SYNC_SLAVE	(1 << 16)

#define DE_P		(0 << 9)
#define DE_N		(1 << 9)

#define PCLK_P		(0 << 10)
#define PCLK_N		(1 << 10)

#define HSYNC_P		(0 << 11)
#define HSYNC_N		(1 << 11)

#define VSYNC_P		(0 << 8)
#define VSYNC_N		(1 << 8)

#define DATA_NORMAL	(0 << 17)
#define DATA_INVERSE	(1 << 17)

#endif /* __JZLCD_H__ */
